/**
 * @class Firework
 */
var FireWork = (function () {
    function FireWork(width, height) {
        var _this = this;
        this.update = function () {
            var retVal = false;
            if (!_this.active)
                return retVal;
            /* check if rocket shall explode (if too slow) */
            if (!_this.exploded && _this.rocket.Speed.Y >= -0.1) {
                retVal = true;
                _this.explode();
            }
            if (!_this.exploded) {
                /* process rocket particle */
                _this.rocket.update();
                /* process smoke particles */
                for (var i = 0; i < _this.smokeParticles.length; i++) {
                    /* correct position of smoke particle so that they follow the movement of the rocket */
                    _this.smokeParticles[i].Speed.addVec(_this.rocket.Acceleration);
                    _this.smokeParticles[i].Position.addVec(_this.rocket.Speed);
                    /* update position, speed and color */
                    _this.smokeParticles[i].update();
                    _this.smokeParticles[i].ColorA *= _this.smokeParticles[i].FadeFactor;
                    /* if smoke particle fades out, replace it with new particle starting from rocket */
                    if (_this.smokeParticles[i].ColorA <= 0.04) {
                        _this.smokeParticles[i].Position.X = _this.rocket.Position.X - 1;
                        _this.smokeParticles[i].Position.Y = _this.rocket.Position.Y + 3;
                        _this.smokeParticles[i].Speed.X = Util.getRandom(-0.58, 0.58);
                        _this.smokeParticles[i].Speed.Y = Util.getRandom(0.55, 1.3);
                        _this.smokeParticles[i].ColorA = 1;
                        _this.smokeParticles[i].FadeFactor = Util.getRandom(0.80, 0.98);
                    }
                }
            }
            else {
                /* process explode particles */
                for (var i = 0; i < _this.explodeParticles.length; i++) {
                    _this.explodeParticles[i].update();
                    _this.explodeParticles[i].Speed.X *= 0.995; /* slow down also in x direction */
                    _this.explodeParticles[i].ColorA *= _this.explodeParticles[i].FadeFactor;
                    /* remove explode particle if faded out or out of visible field of canvas */
                    if (_this.explodeParticles[i].ColorA <= 0.03 || _this.explodeParticles[i].Position.Y > _this.drawAreaHeight) {
                        _this.explodeParticles.splice(i, 1);
                        i--;
                    }
                }
                /* if all particles exploded, set firework to finish state */
                if (_this.explodeParticles.length <= 1) {
                    _this.active = false;
                }
            }
            return retVal;
        };
        /** Display the firework */
        this.display = function (ctx) {
            if (!_this.active)
                return;
            if (!_this.exploded) {
                _this.rocket.display(ctx);
                for (var i = 0; i < _this.smokeParticles.length; i++) {
                    _this.smokeParticles[i].display(ctx);
                }
            }
            else {
                for (var i = 0; i < _this.explodeParticles.length; i++) {
                    _this.explodeParticles[i].display(ctx);
                }
            }
        };
        /** Gets the active state of the firework */
        this.isActive = function () {
            return _this.active;
        };
        /* Explode firework. Remove the smoke particles and create the explode particles */
        this.explode = function () {
            _this.explodeParticles = new Array();
            var explodeParticleRenderer = new ExplodeParticleRenderer();
            /* let the firework explode in several ways */
            var explodeIdx = Util.getRandomInt(0, 2);
            for (var i = 0; i < FireWork.NUM_EXPLODE_PARTICLES; i++) {
                var p = void 0;
                switch (explodeIdx) {
                    case 0:
                        p = new FireworkParticle(new Vector2D(_this.rocket.Position.X, _this.rocket.Position.Y), new Vector2D(Util.getRandom(-1.5, 1.5) + _this.rocket.Speed.X, Util.getRandom(-2.2, 0.4) + +_this.rocket.Speed.Y), new Vector2D(0, 0.04));
                        p.FadeFactor = Util.getRandom(0.985, 0.999);
                        break;
                    case 1:
                        p = new FireworkParticle(new Vector2D(_this.rocket.Position.X, _this.rocket.Position.Y), new Vector2D(Util.getRandom(1.1, 1.6) * Math.cos(2 * Math.PI * FireWork.NUM_EXPLODE_PARTICLES / i), Util.getRandom(1.1, 1.6) * Math.sin(2 * Math.PI * FireWork.NUM_EXPLODE_PARTICLES / i)), new Vector2D(0.007, 0.01));
                        p.FadeFactor = Util.getRandom(0.98, 0.985);
                        break;
                    case 2:
                        p = new FireworkParticle(new Vector2D(_this.rocket.Position.X, _this.rocket.Position.Y), new Vector2D(Util.getRandom(0.5, 1.6) * Math.cos(2 * Math.PI * FireWork.NUM_EXPLODE_PARTICLES / i), Util.getRandom(-0.2, 1.6) * Math.sin(2 * Math.PI * FireWork.NUM_EXPLODE_PARTICLES / i)), new Vector2D(0, 0.04));
                        p.FadeFactor = Util.getRandom(0.985, 0.999);
                        break;
                }
                p.SetColor(_this.rocket.ColorR, _this.rocket.ColorG, _this.rocket.ColorB, _this.rocket.ColorA);
                p.SetRenderFunction(explodeParticleRenderer);
                p.FadeFactor = Util.getRandom(0.98, 0.985);
                _this.explodeParticles.push(p);
            }
            _this.rocket = null;
            _this.smokeParticles = null;
            _this.exploded = true;
        };
        this.drawAreaHeight = height;
        var startPosX = Util.getRandomInt(width * 0.3, width * 0.7);
        var startPosY = height;
        var startSpeedX = Util.getRandom(-1.5, 1.5);
        var startSpeedY = -Util.getRandomInt(9.8, 11.8);
        var gravity = 0.12;
        // create main rocket particle
        this.rocket = new Particle2D(new Vector2D(startPosX, startPosY), new Vector2D(startSpeedX, startSpeedY), new Vector2D(0, gravity));
        var colorIdx = Util.getRandomInt(0, 6);
        switch (colorIdx) {
            case 0:
                this.rocket.SetColor(255, 70, 220, 1);
                break; /* pink */
            case 1:
                this.rocket.SetColor(255, 237, 70, 1);
                break; /* yellow */
            case 2:
                this.rocket.SetColor(255, 30, 30, 1);
                break; /* red */
            case 3:
                this.rocket.SetColor(70, 100, 245, 1);
                break; /* blue */
            case 4:
                this.rocket.SetColor(75, 245, 170, 1);
                break; /* green */
            case 5:
                this.rocket.SetColor(250, 150, 56, 1);
                break; /* orange */
            case 6:
                this.rocket.SetColor(211, 56, 250, 1);
                break; /* purple */
        }
        this.rocket.SetRenderFunction(new RocketParticleRenderer());
        // create smoke particles
        this.smokeParticles = new Array();
        var smokeRenderer = new SmokeParticleRenderer();
        for (var i = 0; i < FireWork.NUM_SMOKE_PARTICLES; i++) {
            var p = new FireworkParticle(new Vector2D(startPosX, startPosY), new Vector2D(-startSpeedX + Util.getRandom(-0.38, 0.38), Util.getRandom(0.55, 1.8)), new Vector2D(0, 0.01));
            p.SetColor(170, 150, 160, 0.4);
            p.SetRenderFunction(smokeRenderer);
            p.FadeFactor = Util.getRandom(0.40, 0.98);
            this.smokeParticles.push(p);
        }
        this.exploded = false;
        this.active = true;
    }
    FireWork.NUM_SMOKE_PARTICLES = 30;
    FireWork.NUM_EXPLODE_PARTICLES = 250;
    return FireWork;
})();
var RocketParticleRenderer = (function () {
    function RocketParticleRenderer() {
        this.displayParticle = function (ctx, particle) {
            ctx.fillStyle = particle.GetColorRGBAString();
            ctx.beginPath();
            ctx.arc(particle.Position.X - 2, particle.Position.Y - 2, 4, 0, 2 * Math.PI, false);
            var cg = ctx.createRadialGradient(particle.Position.X - 2, particle.Position.Y - 2, 0.01, particle.Position.X - 2, particle.Position.Y - 2, 4);
            cg.addColorStop(0, "rgb(255, 255, 255)");
            cg.addColorStop(1, particle.GetColorRGBAString());
            ctx.fillStyle = cg;
            ctx.fill();
        };
    }
    return RocketParticleRenderer;
})();
var SmokeParticleRenderer = (function () {
    function SmokeParticleRenderer() {
        this.displayParticle = function (ctx, particle) {
            ctx.fillStyle = particle.GetColorRGBAString();
            ctx.fillRect(particle.Position.X - 2, particle.Position.Y - 2, 5, 5);
        };
    }
    return SmokeParticleRenderer;
})();
var ExplodeParticleRenderer = (function () {
    function ExplodeParticleRenderer() {
        this.displayParticle = function (ctx, particle) {
            ctx.fillStyle = particle.GetColorRGBAString();
            ctx.fillRect(particle.Position.X - 1, particle.Position.Y - 1, 3, 3);
        };
    }
    return ExplodeParticleRenderer;
})();
//# sourceMappingURL=Firework.js.map